"""
This module gives access to low level bmesh operations.

Most operators take input and return output, they can be chained together
to perform useful operations.


--------------------

This script shows how operators can be used to model a link of a chain.

```__/examples/bmesh.ops.1.py```

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bmesh.types
import bpy.types
import mathutils

def average_vert_facedata(
    bm: bmesh.types.BMesh, verts: list[bmesh.types.BMVert] = []
) -> None:
    """Average Vertices Face-vert Data.Merge uv/vcols associated with the input vertices at
    the bounding box center.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
    """

def beautify_fill(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    edges: list[bmesh.types.BMEdge] = [],
    use_restrict_tag: bool = False,
    method: typing.Literal["AREA", "ANGLE"] = "AREA",
) -> dict[str, typing.Any]:
    """Beautify Fill.Rotate edges to create more evenly spaced triangles.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param edges: Edges that can be flipped.
        :param use_restrict_tag: Restrict edge rotation to mixed tagged vertices.
        :param method: Method to define what is beautiful.
        :return: geom:
    New flipped faces and edges.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def bevel(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    offset: float = 0,
    offset_type: typing.Literal[
        "OFFSET", "WIDTH", "DEPTH", "PERCENT", "ABSOLUTE"
    ] = "OFFSET",
    profile_type: typing.Literal["SUPERELLIPSE", "CUSTOM"] = "SUPERELLIPSE",
    segments: int = 0,
    profile: float = 0,
    affect: typing.Literal["VERTICES", "EDGES"] = "VERTICES",
    clamp_overlap: bool = False,
    material: int = 0,
    loop_slide: bool = False,
    mark_seam: bool = False,
    mark_sharp: bool = False,
    harden_normals: bool = False,
    face_strength_mode: typing.Literal["NONE", "NEW", "AFFECTED", "ALL"] = "NONE",
    miter_outer: typing.Literal["SHARP", "PATCH", "ARC"] = "SHARP",
    miter_inner: typing.Literal["SHARP", "PATCH", "ARC"] = "SHARP",
    spread: float = 0,
    custom_profile: None | bpy.types.bpy_struct | None = None,
    vmesh_method: typing.Literal["ADJ", "CUTOFF"] = "ADJ",
) -> dict[str, typing.Any]:
    """Bevel.Bevels edges and vertices

        :param bm: The bmesh to operate on.
        :param geom: Input edges and vertices.
        :param offset: Amount to offset beveled edge.
        :param offset_type: How to measure the offset.
        :param profile_type: The profile type to use for bevel.
        :param segments: Number of segments in bevel.
        :param profile: Profile shape, 0->1 (.5=>round).
        :param affect: Whether to bevel vertices or edges.
        :param clamp_overlap: Do not allow beveled edges/vertices to overlap each other.
        :param material: Material for bevel faces, -1 means get from adjacent faces.
        :param loop_slide: Prefer to slide along edges to having even widths.
        :param mark_seam: Extend edge data to allow seams to run across bevels.
        :param mark_sharp: Extend edge data to allow sharp edges to run across bevels.
        :param harden_normals: Harden normals.
        :param face_strength_mode: Whether to set face strength, and which faces to set if so.
        :param miter_outer: Outer miter kind.
        :param miter_inner: Inner miter kind.
        :param spread: Amount to spread the miter.
        :param custom_profile: CurveProfile, if None ignored
        :param vmesh_method: The method to use to create meshes at intersections.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]

    edges:
    Output edges.

    type list[`bmesh.types.BMEdge`]

    verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def bisect_edges(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    cuts: int = 0,
    edge_percents: dict[
        bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace, float
    ] = {},
) -> dict[str, typing.Any]:
    """Edge Bisect.Splits input edges (but doesnt do anything else).
    This creates a 2-valence vert.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param cuts: Number of cuts.
        :param edge_percents: Undocumented.
        :return: geom_split:
    Newly created vertices and edges.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def bisect_plane(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    dist: float = 0,
    plane_co: collections.abc.Sequence[float] = mathutils.Vector(),
    plane_no: collections.abc.Sequence[float] = mathutils.Vector(),
    use_snap_center: bool = False,
    clear_outer: bool = False,
    clear_inner: bool = False,
) -> dict[str, typing.Any]:
    """Bisect Plane.Bisects the mesh by a plane (cut the mesh in half).

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param dist: Minimum distance when testing if a vert is exactly on the plane.
        :param plane_co: Point on the plane.
        :param plane_no: Normal of the plane.
        :param use_snap_center: Snap axis aligned verts to the center.
        :param clear_outer: When enabled, remove all geometry on the positive side of the plane.
        :param clear_inner: When enabled, remove all geometry on the negative side of the plane.
        :return: geom_cut:
    Output geometry aligned with the plane (new and existing).

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge`]

    geom:
    Input and output geometry (result of cut).

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def bmesh_to_mesh(
    bm: bmesh.types.BMesh, mesh: bpy.types.Mesh, object: bpy.types.Object
) -> None:
    """BMesh to Mesh.Converts a bmesh to a Mesh. This is reserved for exiting edit-mode.

    :param bm: The bmesh to operate on.
    :param mesh: The mesh to write into.
    :param object: The object.
    """

def bridge_loops(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    use_pairs: bool = False,
    use_cyclic: bool = False,
    use_merge: bool = False,
    merge_factor: float = 0,
    twist_offset: int = 0,
) -> dict[str, typing.Any]:
    """Bridge edge loops with faces.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param use_pairs: Undocumented.
        :param use_cyclic: Undocumented.
        :param use_merge: Merge rather than creating faces.
        :param merge_factor: Merge factor.
        :param twist_offset: Twist offset for closed loops.
        :return: faces:
    New faces.

    type list[`bmesh.types.BMFace`]

    edges:
    New edges.

    type list[`bmesh.types.BMEdge`]
    """

def circularize(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    factor: float = 0,
    custom_radius: float = 0,
    angle: float = 0,
    fit_method: int = 0,
    flatten: float = 0,
    regular: bool = False,
    lock_x: bool = False,
    lock_y: bool = False,
    lock_z: bool = False,
    mirror_x: bool = False,
    mirror_y: bool = False,
    mirror_z: bool = False,
) -> None:
    """Circularize.Shape selected geometry into a circle.

    :param bm: The bmesh to operate on.
    :param geom: Input geometry.
    :param factor: Influence factor: spans from 0.0 to 1.0.
    :param custom_radius: Custom radius.
    :param angle: Rotation angle.
    :param fit_method: Method to fit the circle.
    :param flatten: Flatten factor: 0.0 projects onto the mesh, 1.0 flattens on the optimal plane.
    :param regular: Distributes vertices at constant distances, otherwise preserves original spacing.
    :param lock_x: Lock X-axis editing.
    :param lock_y: Lock Y-axis editing.
    :param lock_z: Lock Z-axis editing.
    :param mirror_x: Use X axis of the mirror modifier.
    :param mirror_y: Use Y axis of the mirror modifier.
    :param mirror_z: Use Z axis of the mirror modifier.
    """

def collapse(
    bm: bmesh.types.BMesh, edges: list[bmesh.types.BMEdge] = [], uvs: bool = False
) -> None:
    """Collapse Connected.Collapses connected vertices

    :param bm: The bmesh to operate on.
    :param edges: Input edges.
    :param uvs: Also collapse UVs and such.
    """

def collapse_uvs(bm: bmesh.types.BMesh, edges: list[bmesh.types.BMEdge] = []) -> None:
    """Collapse Connected UVs.Collapses connected UV vertices.

    :param bm: The bmesh to operate on.
    :param edges: Input edges.
    """

def connect_vert_pair(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    verts_exclude: list[bmesh.types.BMVert] = [],
    faces_exclude: list[bmesh.types.BMFace] = [],
) -> dict[str, typing.Any]:
    """Connect Vert Pair.Connect a pair of vertices by splitting faces along the shortest path between them.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
        :param verts_exclude: Input vertices to explicitly exclude from connecting.
        :param faces_exclude: Input faces to explicitly exclude from connecting.
        :return: edges:

    type list[`bmesh.types.BMEdge`]
    """

def connect_verts(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    faces_exclude: list[bmesh.types.BMFace] = [],
    check_degenerate: bool = False,
) -> dict[str, typing.Any]:
    """Connect Verts.Split faces by adding edges that connect verts.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
        :param faces_exclude: Input faces to explicitly exclude from connecting.
        :param check_degenerate: Prevent splits with overlaps & intersections.
        :return: edges:

    type list[`bmesh.types.BMEdge`]
    """

def connect_verts_concave(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = []
) -> dict[str, typing.Any]:
    """Connect Verts to form Convex Faces.Splits concave faces into convex faces.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :return: edges:

    type list[`bmesh.types.BMEdge`]

    faces:

    type list[`bmesh.types.BMFace`]
    """

def connect_verts_nonplanar(
    bm: bmesh.types.BMesh, angle_limit: float = 0, faces: list[bmesh.types.BMFace] = []
) -> dict[str, typing.Any]:
    """Connect Verts Across non Planar Faces.Split faces by connecting edges along non planar faces.

        :param bm: The bmesh to operate on.
        :param angle_limit: Maximum angle of non-planarity before splitting (radians).
        :param faces: Input faces.
        :return: edges:

    type list[`bmesh.types.BMEdge`]

    faces:

    type list[`bmesh.types.BMFace`]
    """

def contextual_create(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    mat_nr: int = 0,
    use_smooth: bool = False,
) -> dict[str, typing.Any]:
    """Contextual Create.This is basically F-key, it creates
    new faces from vertices, makes stuff from edge nets,
    makes wire edges, etc. It also dissolves faces.Three verts become a triangle, four become a quad. Two
    become a wire edge.

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param mat_nr: Material to use.
        :param use_smooth: Set smooth shading on newly created faces.
        :return: faces:
    Newly-made face(s).

    type list[`bmesh.types.BMFace`]

    edges:
    Newly-made edge(s).

    type list[`bmesh.types.BMEdge`]
    """

def convex_hull(
    bm: bmesh.types.BMesh,
    input: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    use_existing_faces: bool = False,
) -> dict[str, typing.Any]:
    """Convex HullBuilds a convex hull from the vertices in input.If use_existing_faces is true, the hull will not output triangles
    that are covered by a pre-existing face.All hull vertices, faces, and edges are added to geom.out. Any
    input elements that end up inside the hull (i.e. are not used by an
    output face) are added to the geom_interior.out slot. The
    geom_unused.out slot will contain all interior geometry that is
    completely unused. Lastly, geom_holes.out contains edges and faces
    that were in the input and are part of the hull.

        :param bm: The bmesh to operate on.
        :param input: Input geometry.
        :param use_existing_faces: Skip hull triangles that are covered by a pre-existing face.
        :return: geom:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom_interior:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom_unused:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom_holes:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def create_circle(
    bm: bmesh.types.BMesh,
    cap_ends: bool = False,
    cap_tris: bool = False,
    segments: int = 0,
    radius: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Creates a Circle.

        :param bm: The bmesh to operate on.
        :param cap_ends: Whether or not to fill in the circle with a face.
        :param cap_tris: Fill the circle with triangles instead of an n-gon.
        :param segments: Number of vertices in the circle.
        :param radius: Radius of the circle.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_cone(
    bm: bmesh.types.BMesh,
    cap_ends: bool = False,
    cap_tris: bool = False,
    segments: int = 0,
    radius1: float = 0,
    radius2: float = 0,
    depth: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create Cone.Creates a cone with variable radius at both ends

        :param bm: The bmesh to operate on.
        :param cap_ends: Whether or not to fill in the ends with faces.
        :param cap_tris: Fill ends with triangles instead of ngons.
        :param segments: Number of vertices in the base circle.
        :param radius1: Radius of one end.
        :param radius2: Radius of the opposite end.
        :param depth: Distance between ends.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_cube(
    bm: bmesh.types.BMesh,
    size: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create CubeCreates a cube.

        :param bm: The bmesh to operate on.
        :param size: Size of the cube.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_grid(
    bm: bmesh.types.BMesh,
    x_segments: int = 0,
    y_segments: int = 0,
    size: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create Grid.Creates a grid with a variable number of subdivisions

        :param bm: The bmesh to operate on.
        :param x_segments: Number of x segments.
        :param y_segments: Number of y segments.
        :param size: Size of the grid.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_icosphere(
    bm: bmesh.types.BMesh,
    subdivisions: int = 0,
    radius: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create Ico-Sphere.Creates an ico-sphere with a variable number of subdivisions.

        :param bm: The bmesh to operate on.
        :param subdivisions: How many times to recursively subdivide the sphere.
        :param radius: Radius.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_monkey(
    bm: bmesh.types.BMesh,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create Suzanne.Creates a monkey (standard blender primitive).

        :param bm: The bmesh to operate on.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_uvsphere(
    bm: bmesh.types.BMesh,
    u_segments: int = 0,
    v_segments: int = 0,
    radius: float = 0,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    calc_uvs: bool = False,
) -> dict[str, typing.Any]:
    """Create UV Sphere.Creates a UV sphere with a variable number of subdivisions.

        :param bm: The bmesh to operate on.
        :param u_segments: Number of u segments.
        :param v_segments: Number of v segments.
        :param radius: Radius.
        :param matrix: Matrix to multiply the new geometry with.
        :param calc_uvs: Calculate default UVs.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]
    """

def create_vert(
    bm: bmesh.types.BMesh, co: collections.abc.Sequence[float] = mathutils.Vector()
) -> dict[str, typing.Any]:
    """Make Vertex.Creates a single vertex; this BMOP was necessary
    for click-create-vertex.

        :param bm: The bmesh to operate on.
        :param co: The coordinate of the new vert.
        :return: vert:
    The new vert.

    type list[`bmesh.types.BMVert`]
    """

def delete(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    context: typing.Literal[
        "VERTS",
        "EDGES",
        "FACES_ONLY",
        "EDGES_FACES",
        "FACES",
        "FACES_KEEP_BOUNDARY",
        "TAGGED_ONLY",
    ] = "VERTS",
) -> None:
    """Delete Geometry.Utility operator to delete geometry.

    :param bm: The bmesh to operate on.
    :param geom: Input geometry.
    :param context: Geometry types to delete.
    """

def dissolve_degenerate(
    bm: bmesh.types.BMesh, dist: float = 0, edges: list[bmesh.types.BMEdge] = []
) -> None:
    """Degenerate Dissolve.Dissolve edges with no length, faces with no area.

    :param bm: The bmesh to operate on.
    :param dist: Maximum distance to consider degenerate.
    :param edges: Input edges.
    """

def dissolve_edges(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    use_verts: bool = False,
    use_face_split: bool = False,
    angle_threshold: float = 0,
    use_preserve_quads: bool = False,
) -> dict[str, typing.Any]:
    """Dissolve Edges.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param use_verts: Dissolve verts left between only 2 edges.
        :param use_face_split: Split off face corners to maintain surrounding geometry.
        :param angle_threshold: Do not dissolve verts between 2 edges when their angle exceeds this threshold.
    Disabled by default.
        :param use_preserve_quads: When dissolving the edge between 2 triangles, dont dissolve the verts.
        :return: region:

    type list[`bmesh.types.BMFace`]
    """

def dissolve_faces(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = [], use_verts: bool = False
) -> dict[str, typing.Any]:
    """Dissolve Faces.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param use_verts: Dissolve verts left between only 2 edges.
        :return: region:

    type list[`bmesh.types.BMFace`]
    """

def dissolve_limit(
    bm: bmesh.types.BMesh,
    angle_limit: float = 0,
    use_dissolve_boundaries: bool = False,
    verts: list[bmesh.types.BMVert] = [],
    edges: list[bmesh.types.BMEdge] = [],
    delimit: set[typing.Literal["NORMAL", "MATERIAL", "SEAM", "SHARP", "UV"]] = set(),
) -> dict[str, typing.Any]:
    """Limited Dissolve.Dissolve planar faces and co-linear edges.

        :param bm: The bmesh to operate on.
        :param angle_limit: Maximum angle (radians) between face normals for dissolving.
        :param use_dissolve_boundaries: Dissolve all vertices in between face boundaries.
        :param verts: Input vertices.
        :param edges: Input edges.
        :param delimit: Delimit dissolve operation.
        :return: region:

    type list[`bmesh.types.BMFace`]
    """

def dissolve_verts(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    use_face_split: bool = False,
    use_boundary_tear: bool = False,
) -> None:
    """Dissolve Verts.

    :param bm: The bmesh to operate on.
    :param verts: Input vertices.
    :param use_face_split: Split off face corners to maintain surrounding geometry.
    :param use_boundary_tear: Split off face corners instead of merging faces.
    """

def duplicate(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    dest: None | bmesh.types.BMesh | None = None,
    use_select_history: bool = False,
    use_edge_flip_from_face: bool = False,
) -> dict[str, typing.Any]:
    """Duplicate Geometry.Utility operator to duplicate geometry,
    optionally into a destination mesh.

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param dest: Destination bmesh, if None will use current one.
        :param use_select_history: Preserve the selection history in the duplicated geometry.
        :param use_edge_flip_from_face: Copy edge flip state from connected faces.
        :return: geom_orig:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    vert_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    edge_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    face_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    boundary_map:
    Boundary edges from the split geometry that maps edges from the original geometry
    to the destination edges.

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    isovert_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def edgeloop_fill(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    mat_nr: int = 0,
    use_smooth: bool = False,
) -> dict[str, typing.Any]:
    """Edge Loop Fill.Create faces defined by one or more non overlapping edge loops.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param mat_nr: Material to use.
        :param use_smooth: Smooth state to use.
        :return: faces:
    New faces.

    type list[`bmesh.types.BMFace`]
    """

def edgenet_fill(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    mat_nr: int = 0,
    use_smooth: bool = False,
    sides: int = 0,
) -> dict[str, typing.Any]:
    """Edge Net Fill.Create faces defined by enclosed edges.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param mat_nr: Material to use.
        :param use_smooth: Smooth state to use.
        :param sides: Maximum number of sides for created faces.
        :return: faces:
    New faces.

    type list[`bmesh.types.BMFace`]
    """

def edgenet_prepare(
    bm: bmesh.types.BMesh, edges: list[bmesh.types.BMEdge] = []
) -> dict[str, typing.Any]:
    """Edge-net Prepare.Identifies several useful edge loop cases and modifies them so
    theyll become a face when edgenet_fill is called. The cases covered are:

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :return: edges:
    New edges.

    type list[`bmesh.types.BMEdge`]
    """

def extrude_discrete_faces(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    use_normal_flip: bool = False,
    use_select_history: bool = False,
) -> dict[str, typing.Any]:
    """Individual Face Extrude.Extrudes faces individually.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param use_normal_flip: Create faces with reversed direction.
        :param use_select_history: Preserve the selection history in the extruded geometry.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """

def extrude_edge_only(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    use_normal_flip: bool = False,
    use_select_history: bool = False,
) -> dict[str, typing.Any]:
    """Extrude Only Edges.Extrudes Edges into faces, note that this is very simple, theres no fancy
    winged extrusion.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param use_normal_flip: Create faces with reversed direction.
        :param use_select_history: Preserve the selection history in the extruded geometry.
        :return: geom:
    Output geometry.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def extrude_face_region(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    edges_exclude: set[
        bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace
    ] = set(),
    use_keep_orig: bool = False,
    use_normal_flip: bool = False,
    use_normal_from_adjacent: bool = False,
    use_dissolve_ortho_edges: bool = False,
    use_select_history: bool = False,
    skip_input_flip: bool = False,
) -> dict[str, typing.Any]:
    """Extrude Faces.Extrude operator (does not transform)

        :param bm: The bmesh to operate on.
        :param geom: Edges and faces.
        :param edges_exclude: Input edges to explicitly exclude from extrusion.
        :param use_keep_orig: Keep original geometry (requires geom to include edges).
        :param use_normal_flip: Create faces with reversed direction.
        :param use_normal_from_adjacent: Use winding from surrounding faces instead of this region.
        :param use_dissolve_ortho_edges: Dissolve edges whose faces form a flat surface.
        :param use_select_history: Preserve the selection history in the extruded geometry.
        :param skip_input_flip: Skip flipping of input faces to preserve original orientation.
        :return: geom:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def extrude_vert_indiv(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    use_select_history: bool = False,
) -> dict[str, typing.Any]:
    """Individual Vertex Extrude.Extrudes individual vertices, creating new vertices connected by wire edges.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
        :param use_select_history: Preserve the selection history in the extruded geometry.
        :return: edges:
    Output wire edges.

    type list[`bmesh.types.BMEdge`]

    verts:
    Output vertices.

    type list[`bmesh.types.BMVert`]
    """

def face_attribute_fill(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    use_normals: bool = False,
    use_data: bool = False,
) -> dict[str, typing.Any]:
    """Face Attribute Fill.Fill in faces with data from adjacent faces.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param use_normals: Copy face winding.
        :param use_data: Copy face data.
        :return: faces_fail:
    Faces that could not be handled.

    type list[`bmesh.types.BMFace`]
    """

def find_doubles(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    keep_verts: list[bmesh.types.BMVert] = [],
    use_connected: bool = False,
    dist: float = 0,
) -> dict[str, typing.Any]:
    """Find Doubles.Takes input verts and finds vertices they should weld to.
    Outputs a mapping slot suitable for use with the weld verts BMOP.If keep_verts is used, vertices outside that set can only be merged
    with vertices in that set.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
        :param keep_verts: List of verts to keep.
        :param use_connected: Limit the search for doubles by connected geometry.
        :param dist: Maximum distance.
        :return: targetmap:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def flip_quad_tessellation(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = []
) -> None:
    """Flip Quad TessellationFlip the tessellation direction of the selected quads.

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    """

def grid_fill(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    mat_nr: int = 0,
    use_smooth: bool = False,
    use_interp_simple: bool = False,
) -> dict[str, typing.Any]:
    """Grid Fill.Create faces defined by 2 disconnected edge loops (which share edges).

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param mat_nr: Material to use.
        :param use_smooth: Smooth state to use.
        :param use_interp_simple: Use simple interpolation.
        :return: faces:
    New faces.

    type list[`bmesh.types.BMFace`]
    """

def holes_fill(
    bm: bmesh.types.BMesh, edges: list[bmesh.types.BMEdge] = [], sides: int = 0
) -> dict[str, typing.Any]:
    """Fill Holes.Fill boundary edges with faces, copying surrounding custom-data.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param sides: Maximum number of sides for holes to fill (holes with more edges are skipped).
        :return: faces:
    New faces.

    type list[`bmesh.types.BMFace`]
    """

def inset_individual(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    thickness: float = 0,
    depth: float = 0,
    use_even_offset: bool = False,
    use_interpolate: bool = False,
    use_relative_offset: bool = False,
) -> dict[str, typing.Any]:
    """Face Inset (Individual).Insets individual faces.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param thickness: Inset distance from the boundary.
        :param depth: Distance to raise or lower the inset face along its normal.
        :param use_even_offset: Scale the offset to give more even thickness.
        :param use_interpolate: Blend face data across the inset.
        :param use_relative_offset: Scale the offset by surrounding geometry.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """

def inset_region(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    faces_exclude: list[bmesh.types.BMFace] = [],
    use_boundary: bool = False,
    use_even_offset: bool = False,
    use_interpolate: bool = False,
    use_relative_offset: bool = False,
    use_edge_rail: bool = False,
    thickness: float = 0,
    depth: float = 0,
    use_outset: bool = False,
) -> dict[str, typing.Any]:
    """Face Inset (Regions).Inset or outset face regions.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param faces_exclude: Input faces to explicitly exclude from inset.
        :param use_boundary: Inset face boundaries.
        :param use_even_offset: Scale the offset to give more even thickness.
        :param use_interpolate: Blend face data across the inset.
        :param use_relative_offset: Scale the offset by surrounding geometry.
        :param use_edge_rail: Inset the region along existing edges.
        :param thickness: Inset distance from the boundary.
        :param depth: Distance to raise or lower the inset face along its normal.
        :param use_outset: Outset rather than inset.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """

def join_triangles(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    cmp_seam: bool = False,
    cmp_sharp: bool = False,
    cmp_uvs: bool = False,
    cmp_vcols: bool = False,
    cmp_materials: bool = False,
    angle_face_threshold: float = 0,
    angle_shape_threshold: float = 0,
    topology_influence: float = 0,
    deselect_joined: bool = False,
    merge_limit: int = 0,
    neighbor_debug: int = 0,
) -> dict[str, typing.Any]:
    """Join Triangles.Tries to intelligently join triangles according
    to angle threshold and delimiters.

        :param bm: The bmesh to operate on.
        :param faces: Input geometry.
        :param cmp_seam: Compare seam
        :param cmp_sharp: Compare sharp
        :param cmp_uvs: Compare UVs
        :param cmp_vcols: Compare VCols.
        :param cmp_materials: Compare materials.
        :param angle_face_threshold: Undocumented.
        :param angle_shape_threshold: Undocumented.
        :param topology_influence: Undocumented.
        :param deselect_joined: Undocumented.
        :param merge_limit: Undocumented.
        :param neighbor_debug: Undocumented.
        :return: faces:
    Joined faces.

    type list[`bmesh.types.BMFace`]
    """

def mesh_to_bmesh(
    bm: bmesh.types.BMesh,
    mesh: bpy.types.Mesh,
    object: bpy.types.Object,
    use_shapekey: bool = False,
) -> None:
    """Mesh to BMesh.Load the contents of a mesh into the bmesh. this BMOP is private, its
    reserved exclusively for entering edit-mode.

        :param bm: The bmesh to operate on.
        :param mesh: The mesh to read from.
        :param object: The object.
        :param use_shapekey: Load active shapekey coordinates into verts.
    """

def mirror(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    merge_dist: float = 0,
    axis: typing.Literal["X", "Y", "Z"] = "X",
    mirror_u: bool = False,
    mirror_v: bool = False,
    mirror_udim: bool = False,
    use_shapekey: bool = False,
) -> dict[str, typing.Any]:
    """Mirror.Mirrors geometry along an axis. The resulting geometry is welded on using
    merge_dist. Pairs of original/mirrored vertices are welded using the merge_dist
    parameter (which defines the minimum distance for welding to happen).

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param matrix: Matrix defining the mirror transformation.
        :param merge_dist: Maximum distance for merging. does no merging if 0.
        :param axis: The axis to use.
        :param mirror_u: Mirror UVs across the u axis.
        :param mirror_v: Mirror UVs across the v axis.
        :param mirror_udim: Mirror UVs in each tile.
        :param use_shapekey: Transform shape keys too.
        :return: geom:
    Output geometry, mirrored.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def object_load_bmesh(
    bm: bmesh.types.BMesh, scene: bpy.types.Scene, object: bpy.types.Object
) -> None:
    """Object Load BMesh.Loads a bmesh into an object/mesh. This is a "private"
    BMOP.

        :param bm: The bmesh to operate on.
        :param scene: The scene.
        :param object: The object.
    """

def offset_edgeloops(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    use_cap_endpoint: bool = False,
) -> dict[str, typing.Any]:
    """Edge-loop Offset.Creates edge loops based on simple edge-outset method.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param use_cap_endpoint: Extend loop around end-points.
        :return: edges:
    Output edges.

    type list[`bmesh.types.BMEdge`]
    """

def planar_faces(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    iterations: int = 0,
    factor: float = 0,
) -> dict[str, typing.Any]:
    """Planar Faces.Iteratively flatten faces.

        :param bm: The bmesh to operate on.
        :param faces: Input geometry.
        :param iterations: Number of times to flatten faces (for when connected faces are used)
        :param factor: Influence for making planar each iteration
        :return: geom:
    Output slot, computed boundary geometry.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def pointmerge(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    merge_co: collections.abc.Sequence[float] = mathutils.Vector(),
) -> None:
    """Point Merge.Merge verts together at a point.

    :param bm: The bmesh to operate on.
    :param verts: Input vertices (all verts will be merged into the first).
    :param merge_co: Position to merge at.
    """

def pointmerge_facedata(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    vert_snap: None | bmesh.types.BMVert | None = None,
) -> None:
    """Face-Data Point Merge.Merge uv/vcols at a specific vertex.

    :param bm: The bmesh to operate on.
    :param verts: Input vertices.
    :param vert_snap: Snap vertex.
    """

def poke(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    offset: float = 0,
    center_mode: typing.Literal["MEAN_WEIGHTED", "MEAN", "BOUNDS"] = "MEAN_WEIGHTED",
    use_relative_offset: bool = False,
) -> dict[str, typing.Any]:
    """Pokes a face.Splits a face into a triangle fan.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param offset: Center vertex offset along normal.
        :param center_mode: Calculation mode for center vertex.
        :param use_relative_offset: Apply offset.
        :return: verts:
    Output verts.

    type list[`bmesh.types.BMVert`]

    faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """

def recalc_face_normals(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = []
) -> None:
    """Right-Hand Faces.Computes an "outside" normal for the specified input faces.

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    """

def region_extend(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    use_contract: bool = False,
    use_faces: bool = False,
    use_face_step: bool = False,
) -> dict[str, typing.Any]:
    """Region Extend.Used to implement the select more/less tools.
    Puts geometry surrounding regions of geometry in geom into geom.out.If use_faces is 0 then geom.out spits out verts and edges,
    otherwise it spits out faces.

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param use_contract: Find boundary inside the regions, not outside.
        :param use_faces: Extend from faces instead of edges.
        :param use_face_step: Step over connected faces.
        :return: geom:
    Output slot, computed boundary geometry.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def remove_doubles(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    use_connected: bool = False,
    dist: float = 0,
) -> None:
    """Remove Doubles.Finds groups of vertices closer than dist and merges them together,
    using the weld verts BMOP.

        :param bm: The bmesh to operate on.
        :param verts: Input verts.
        :param use_connected: Limit the search for doubles by connected geometry.
        :param dist: Maximum distance.
    """

def reverse_colors(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = [], color_index: int = 0
) -> None:
    """Color ReverseReverse the loop colors.

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    :param color_index: Index into color attribute list.
    """

def reverse_faces(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    flip_multires: bool = False,
) -> None:
    """Reverse Faces.Reverses the winding (vertex order) of faces.
    This has the effect of flipping the normal.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param flip_multires: Maintain multi-res offset.
    """

def reverse_uvs(bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = []) -> None:
    """UV Reverse.Reverse the UVs

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    """

def rotate(
    bm: bmesh.types.BMesh,
    cent: collections.abc.Sequence[float] = mathutils.Vector(),
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    verts: list[bmesh.types.BMVert] = [],
    space: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    use_shapekey: bool = False,
) -> None:
    """Rotate.Rotate vertices around a center, using a 3x3 rotation matrix.

    :param bm: The bmesh to operate on.
    :param cent: Center of rotation.
    :param matrix: Matrix defining rotation.
    :param verts: Input vertices.
    :param space: Matrix to define the space (typically object matrix).
    :param use_shapekey: Transform shape keys too.
    """

def rotate_colors(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    use_ccw: bool = False,
    color_index: int = 0,
) -> None:
    """Color Rotation.Cycle the loop colors

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    :param use_ccw: Rotate counter-clockwise if true, otherwise clockwise.
    :param color_index: Index into color attribute list.
    """

def rotate_edges(
    bm: bmesh.types.BMesh, edges: list[bmesh.types.BMEdge] = [], use_ccw: bool = False
) -> dict[str, typing.Any]:
    """Edge Rotate.Rotates edges topologically. Also known as "spin edge" to some people.
    Simple example: [/] becomes [|] then [\].

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param use_ccw: Rotate edge counter-clockwise if true, otherwise clockwise.
        :return: edges:
    Newly spun edges.

    type list[`bmesh.types.BMEdge`]
    """

def rotate_uvs(
    bm: bmesh.types.BMesh, faces: list[bmesh.types.BMFace] = [], use_ccw: bool = False
) -> None:
    """UV Rotation.Cycle the loop UVs

    :param bm: The bmesh to operate on.
    :param faces: Input faces.
    :param use_ccw: Rotate counter-clockwise if true, otherwise clockwise.
    """

def scale(
    bm: bmesh.types.BMesh,
    vec: collections.abc.Sequence[float] = mathutils.Vector(),
    space: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    verts: list[bmesh.types.BMVert] = [],
    use_shapekey: bool = False,
) -> None:
    """Scale.Scales vertices by a factor.

    :param bm: The bmesh to operate on.
    :param vec: Scale factor.
    :param space: Matrix to define the space (typically object matrix).
    :param verts: Input vertices.
    :param use_shapekey: Transform shape keys too.
    """

def smooth_laplacian_vert(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    lambda_factor: float = 0,
    lambda_border: float = 0,
    use_x: bool = False,
    use_y: bool = False,
    use_z: bool = False,
    preserve_volume: bool = False,
) -> None:
    """Vertex Smooth Laplacian.Smooths vertices by using Laplacian smoothing proposed by
    Desbrun, et al. Implicit Fairing of Irregular Meshes using Diffusion and Curvature Flow.

        :param bm: The bmesh to operate on.
        :param verts: Input vertices.
        :param lambda_factor: Lambda parameter.
        :param lambda_border: Lambda param in border.
        :param use_x: Smooth object along X axis.
        :param use_y: Smooth object along Y axis.
        :param use_z: Smooth object along Z axis.
        :param preserve_volume: Apply volume preservation after smooth.
    """

def smooth_vert(
    bm: bmesh.types.BMesh,
    verts: list[bmesh.types.BMVert] = [],
    factor: float = 0,
    mirror_clip_x: bool = False,
    mirror_clip_y: bool = False,
    mirror_clip_z: bool = False,
    clip_dist: float = 0,
    use_axis_x: bool = False,
    use_axis_y: bool = False,
    use_axis_z: bool = False,
) -> None:
    """Vertex Smooth.Smooths vertices by using a basic vertex averaging scheme.

    :param bm: The bmesh to operate on.
    :param verts: Input vertices.
    :param factor: Smoothing factor.
    :param mirror_clip_x: Set vertices close to the x axis before the operation to 0.
    :param mirror_clip_y: Set vertices close to the y axis before the operation to 0.
    :param mirror_clip_z: Set vertices close to the z axis before the operation to 0.
    :param clip_dist: Clipping threshold for the above three slots.
    :param use_axis_x: Smooth vertices along X axis.
    :param use_axis_y: Smooth vertices along Y axis.
    :param use_axis_z: Smooth vertices along Z axis.
    """

def solidify(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    thickness: float = 0,
) -> dict[str, typing.Any]:
    """Solidify.Turns a mesh into a shell with thickness

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param thickness: Thickness of the solidified shell.
        :return: geom:
    Output geometry (new shell faces, edges, and vertices).

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def spin(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    cent: collections.abc.Sequence[float] = mathutils.Vector(),
    axis: collections.abc.Sequence[float] = mathutils.Vector(),
    dvec: collections.abc.Sequence[float] = mathutils.Vector(),
    angle: float = 0,
    space: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    steps: int = 0,
    use_merge: bool = False,
    use_normal_flip: bool = False,
    use_duplicate: bool = False,
) -> dict[str, typing.Any]:
    """Spin.Extrude or duplicate geometry a number of times,
    rotating and possibly translating after each step

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param cent: Rotation center.
        :param axis: Rotation axis.
        :param dvec: Translation delta per step.
        :param angle: Total rotation angle (radians).
        :param space: Matrix to define the space (typically object matrix).
        :param steps: Number of steps.
        :param use_merge: Merge first/last when the angle is a full revolution.
        :param use_normal_flip: Create faces with reversed direction.
        :param use_duplicate: Duplicate the geometry, otherwise extrude.
        :return: geom_last:
    Result of last step.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def split(
    bm: bmesh.types.BMesh,
    geom: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    dest: None | bmesh.types.BMesh | None = None,
    use_only_faces: bool = False,
) -> dict[str, typing.Any]:
    """Split Off Geometry.Disconnect geometry from adjacent edges and faces,
    optionally into a destination mesh.

        :param bm: The bmesh to operate on.
        :param geom: Input geometry.
        :param dest: Destination bmesh, if None will use current one.
        :param use_only_faces: When enabled, dont duplicate loose verts/edges.
        :return: geom:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    boundary_map:
    Boundary edges from the split geometry that maps edges from the original geometry
    to the destination edges.

    When the source edges have been deleted, the destination edge will be used
    for both the key and the value.

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    isovert_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def split_edges(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    verts: list[bmesh.types.BMVert] = [],
    use_verts: bool = False,
) -> dict[str, typing.Any]:
    """Edge Split.Disconnects faces along input edges.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param verts: Optional tag verts, use to have greater control of splits.
        :param use_verts: Use verts for splitting, else just find verts to split from edges.
        :return: edges:
    The original edges that were disconnected.

    type list[`bmesh.types.BMEdge`]
    """

def subdivide_edgering(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    interp_mode: typing.Literal["LINEAR", "PATH", "SURFACE"] = "LINEAR",
    smooth: float = 0,
    cuts: int = 0,
    profile_shape: typing.Literal[
        "SMOOTH", "SPHERE", "ROOT", "SHARP", "LINEAR", "INVERSE_SQUARE"
    ] = "SMOOTH",
    profile_shape_factor: float = 0,
) -> dict[str, typing.Any]:
    """Subdivide Edge-Ring.Take an edge-ring, and subdivide with interpolation options.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param interp_mode: Interpolation method.
        :param smooth: Smoothness factor.
        :param cuts: Number of cuts.
        :param profile_shape: Profile shape type.
        :param profile_shape_factor: How much intermediary new edges are shrunk/expanded.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """

def subdivide_edges(
    bm: bmesh.types.BMesh,
    edges: list[bmesh.types.BMEdge] = [],
    smooth: float = 0,
    smooth_falloff: typing.Literal[
        "SMOOTH", "SPHERE", "ROOT", "SHARP", "LINEAR", "INVERSE_SQUARE"
    ] = "SMOOTH",
    fractal: float = 0,
    along_normal: float = 0,
    cuts: int = 0,
    seed: int = 0,
    custom_patterns: dict = {},
    edge_percents: dict[
        bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace, float
    ] = {},
    quad_corner_type: typing.Literal[
        "STRAIGHT_CUT", "INNER_VERT", "PATH", "FAN"
    ] = "STRAIGHT_CUT",
    use_grid_fill: bool = False,
    use_single_edge: bool = False,
    use_only_quads: bool = False,
    use_sphere: bool = False,
    use_smooth_even: bool = False,
) -> dict[str, typing.Any]:
    """Subdivide Edges.Advanced operator for subdividing edges
    with options for face patterns, smoothing and randomization.

        :param bm: The bmesh to operate on.
        :param edges: Input edges.
        :param smooth: Smoothness factor.
        :param smooth_falloff: Smooth falloff type.
        :param fractal: Fractal randomness factor.
        :param along_normal: Factor (0 to 1) controlling how much fractal displacement is restricted to the normal.
        :param cuts: Number of cuts.
        :param seed: Seed for the random number generator.
        :param custom_patterns: Internal use only, not accessible from Python.
        :param edge_percents: Mapping of edges to a float (0 to 1) controlling the cut position along each edge.
        :param quad_corner_type: Quad corner type.
        :param use_grid_fill: Fill in fully-selected faces with a grid.
        :param use_single_edge: Tessellate the case of one edge selected in a quad or triangle.
        :param use_only_quads: Only subdivide quads (for loop-cut).
        :param use_sphere: Project new vertices onto a sphere (used for spherical primitives).
        :param use_smooth_even: Maintain even offset when smoothing.
        :return: geom_inner:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom_split:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    geom:
    Contains all output geometry.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def symmetrize(
    bm: bmesh.types.BMesh,
    input: list[bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace] = [],
    direction: typing.Literal["-X", "-Y", "-Z", "X", "Y", "Z"] = "-X",
    dist: float = 0,
    use_shapekey: bool = False,
) -> dict[str, typing.Any]:
    """Symmetrize.Makes the mesh elements in the input slot symmetrical. Unlike
    normal mirroring, it only copies in one direction, as specified by
    the direction slot. The edges and faces that cross the plane of
    symmetry are split as needed to enforce symmetry.All new vertices, edges, and faces are added to the geom.out slot.

        :param bm: The bmesh to operate on.
        :param input: Input geometry.
        :param direction: Axis to use.
        :param dist: Minimum distance.
        :param use_shapekey: Transform shape keys too.
        :return: geom:

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def transform(
    bm: bmesh.types.BMesh,
    matrix: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    space: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    verts: list[bmesh.types.BMVert] = [],
    use_shapekey: bool = False,
) -> None:
    """Transform.Transforms a set of vertices by a matrix. Multiplies
    the vertex coordinates with the matrix.

        :param bm: The bmesh to operate on.
        :param matrix: Transform matrix.
        :param space: Matrix to define the space (typically object matrix).
        :param verts: Input vertices.
        :param use_shapekey: Transform shape keys too.
    """

def translate(
    bm: bmesh.types.BMesh,
    vec: collections.abc.Sequence[float] = mathutils.Vector(),
    space: collections.abc.Sequence[collections.abc.Sequence[float]]
    | mathutils.Matrix = mathutils.Matrix.Identity(4),
    verts: list[bmesh.types.BMVert] = [],
    use_shapekey: bool = False,
) -> None:
    """Translate.Translate vertices by an offset.

    :param bm: The bmesh to operate on.
    :param vec: Translation offset.
    :param space: Matrix to define the space (typically object matrix).
    :param verts: Input vertices.
    :param use_shapekey: Transform shape keys too.
    """

def triangle_fill(
    bm: bmesh.types.BMesh,
    use_beauty: bool = False,
    use_dissolve: bool = False,
    edges: list[bmesh.types.BMEdge] = [],
    normal: collections.abc.Sequence[float] = mathutils.Vector(),
) -> dict[str, typing.Any]:
    """Triangle Fill.Fill edges with triangles

        :param bm: The bmesh to operate on.
        :param use_beauty: Use best triangulation division.
        :param use_dissolve: Dissolve resulting faces.
        :param edges: Input edges.
        :param normal: Optionally pass the fill normal to use.
        :return: geom:
    New faces and edges.

    type list[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def triangulate(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    quad_method: typing.Literal[
        "BEAUTY", "FIXED", "ALTERNATE", "SHORT_EDGE", "LONG_EDGE"
    ] = "BEAUTY",
    ngon_method: typing.Literal["BEAUTY", "EAR_CLIP"] = "BEAUTY",
) -> dict[str, typing.Any]:
    """Triangulate.Triangulate faces, splitting quads and n-gons into triangles.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param quad_method: Method for splitting the quads into triangles.
        :param ngon_method: Method for splitting the polygons into triangles.
        :return: edges:

    type list[`bmesh.types.BMEdge`]

    faces:

    type list[`bmesh.types.BMFace`]

    face_map:

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]

    face_map_double:
    Duplicate faces.

    type dict[`bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`, `bmesh.types.BMVert` | `bmesh.types.BMEdge` | `bmesh.types.BMFace`]
    """

def unsubdivide(
    bm: bmesh.types.BMesh, verts: list[bmesh.types.BMVert] = [], iterations: int = 0
) -> None:
    """Un-Subdivide.Reduce detail in geometry containing grids.

    :param bm: The bmesh to operate on.
    :param verts: Input vertices.
    :param iterations: Number of times to unsubdivide.
    """

def weld_verts(
    bm: bmesh.types.BMesh,
    targetmap: dict[
        bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace,
        bmesh.types.BMVert | bmesh.types.BMEdge | bmesh.types.BMFace,
    ] = {},
    use_centroid: bool = False,
) -> None:
    """Weld Verts.Welds verts together (kind-of like remove doubles, merge, etc, all of which
    use or will use this BMOP). You pass in mappings from vertices to the vertices
    they weld with.

        :param bm: The bmesh to operate on.
        :param targetmap: Maps welded vertices to verts they should weld to.
        :param use_centroid: Merge vertices to their centroid position,
    otherwise use the position of the target vertex.
    """

def wireframe(
    bm: bmesh.types.BMesh,
    faces: list[bmesh.types.BMFace] = [],
    thickness: float = 0,
    offset: float = 0,
    use_replace: bool = False,
    use_boundary: bool = False,
    use_even_offset: bool = False,
    use_crease: bool = False,
    crease_weight: float = 0,
    use_relative_offset: bool = False,
    material_offset: int = 0,
) -> dict[str, typing.Any]:
    """Wire Frame.Makes a wire-frame copy of faces.

        :param bm: The bmesh to operate on.
        :param faces: Input faces.
        :param thickness: Wire thickness.
        :param offset: Offset the thickness from the center.
        :param use_replace: Remove original geometry.
        :param use_boundary: Inset face boundaries.
        :param use_even_offset: Scale the offset to give more even thickness.
        :param use_crease: Crease hub edges for improved subdivision surface.
        :param crease_weight: The mean crease weight for resulting edges.
        :param use_relative_offset: Scale the offset by surrounding geometry.
        :param material_offset: Offset material index of generated faces.
        :return: faces:
    Output faces.

    type list[`bmesh.types.BMFace`]
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def convert_bricks(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Convert Bricks to Nodes

    :return: Result of the operator call.
    """

def duplicate_brick(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Duplicate this brick

    :return: Result of the operator call.
    """

def remove_actuator(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    target_brick: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove the selected actuator from the selected object

    :param target_brick: target_brick, (optional, never None)
    :return: Result of the operator call.
    """

def remove_controller(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    target_brick: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove the selected controller from the selected object

    :param target_brick: target_brick, (optional, never None)
    :return: Result of the operator call.
    """

def remove_sensor(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    target_brick: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove the selected sensor from the selected object

    :param target_brick: target_brick, (optional, never None)
    :return: Result of the operator call.
    """

def update_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Synchronize logic bricks with the node setup. This should normally happen automatically

    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

class BHead4:
    """BHead4(code: bytes, len: int, old: int, SDNAnr: int, nr: int)"""

class BlendFileHeader:
    """BlendFileHeader represents the first 12-17 bytes of a blend file.It contains information about the hardware architecture, which is relevant
    to the structure of the rest of the file.
    """

    def create_block_header_struct(self) -> None: ...

class BlendHeaderError:
    """Common base class for all non-exit exceptions."""

    args: typing.Any

class BlockHeader:
    """A .blend file consists of a sequence of blocks whereby each block has a header.
    This class can parse a header block in a specific .blend file.Note the binary representation of this header is different for different files.
    This class provides a unified interface for these underlying representations.
    """

    addr_old: typing.Any
    code: typing.Any
    count: typing.Any
    sdna_index: typing.Any
    size: typing.Any

class BlockHeaderStruct:
    """BlockHeaderStruct(struct: _struct.Struct, type: Type[Union[_blendfile_header.BHead4, _blendfile_header.SmallBHead8, _blendfile_header.LargeBHead8]])"""

    size: typing.Any

    def parse(self, data) -> None:
        """

        :param data:
        """

class LargeBHead8:
    """LargeBHead8(code: bytes, SDNAnr: int, old: int, len: int, nr: int)"""

class SmallBHead8:
    """SmallBHead8(code: bytes, len: int, old: int, SDNAnr: int, nr: int)"""



"""
Module to access application values that remain unchanged during runtime.

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

has_joystick: bool
""" True if the BGE has been built with joystick support.
"""

has_physics: bool
""" True if the BGE has been built with physics support.
"""

has_texture_ffmpeg: bool
""" True if the BGE has been built with FFmpeg support,
enabling use of `~bge.texture.ImageFFmpeg` and `~bge.texture.VideoFFmpeg`.
"""

upbge_version: typing.Any
""" The UPBGE version as a tuple of 3 ints, eg. (0, 0, 3).
"""

upbge_version_string: str
""" The UPBGE version formatted as a string, eg. "0.0 (sub 3)".
"""

version: typing.Any
""" The Blender/BGE version as a tuple of 3 ints, eg. (2, 75, 1).
"""

version_char: str
""" The Blender/BGE version character (for minor releases).
"""

version_string: str
""" The Blender/BGE version formatted as a string, eg. "2.75 (sub 1)".
"""



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def bake_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Entire Fluid Simulation

    :return: Result of the operator call.
    """

def bake_data(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Fluid Data

    :return: Result of the operator call.
    """

def bake_guides(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Fluid Guiding

    :return: Result of the operator call.
    """

def bake_mesh(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Fluid Mesh

    :return: Result of the operator call.
    """

def bake_noise(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Fluid Noise

    :return: Result of the operator call.
    """

def bake_particles(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bake Fluid Particles

    :return: Result of the operator call.
    """

def free_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Entire Fluid Simulation

    :return: Result of the operator call.
    """

def free_data(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Fluid Data

    :return: Result of the operator call.
    """

def free_guides(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Fluid Guiding

    :return: Result of the operator call.
    """

def free_mesh(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Fluid Mesh

    :return: Result of the operator call.
    """

def free_noise(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Fluid Noise

    :return: Result of the operator call.
    """

def free_particles(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Free Fluid Particles

    :return: Result of the operator call.
    """

def pause_bake(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Pause Bake

    :return: Result of the operator call.
    """

def preset_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    name: str | None = "",
    remove_name: bool | None = False,
    remove_active: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add or remove a Fluid Preset

    :param name: Name, Name of the preset, used to make the path name (optional, never None)
    :param remove_name: remove_name, (optional)
    :param remove_active: remove_active, (optional)
    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def add_component(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    component: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a python Component to the selected object

    :param component: Component Name, Add this Component to the current object (optional)
    :return: Result of the operator call.
    """

def add_game_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Adds a property available to the UPBGE

    :return: Result of the operator call.
    """

def add_global_category(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a global value category

    :return: Result of the operator call.
    """

def add_global_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a value accessible from anywhere

    :return: Result of the operator call.
    """

def add_logic_tree_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Adds a property available to the UPBGE

    :return: Result of the operator call.
    """

def add_portal_in(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    portal_name: str | None = "Portal",
    mode: typing.Literal[
        "0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12", "13", "14"
    ]
    | None = "1",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Create a new portal

    :param portal_name: Portal Name, (optional, never None)
    :param mode: Socket Type, (optional)
    :return: Result of the operator call.
    """

def add_portal_out(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    portal_name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Create a new portal

    :param portal_name: Portal Name, (optional, never None)
    :return: Result of the operator call.
    """

def add_socket(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    socket_type: str | None = "NLListItemSocket",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a socket to this node

    :param socket_type: socket_type, (optional, never None)
    :return: Result of the operator call.
    """

def add_template(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    nl_template_name: str | None = "",
    owner: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a template

    :param nl_template_name: nl_template_name, (optional, never None)
    :param owner: owner, (optional, never None)
    :return: Result of the operator call.
    """

def apply_logic_tree(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    owner: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Apply the current tree to the selected objects.

    :param owner: owner, (optional, never None)
    :return: Result of the operator call.
    """

def audio_system(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select the object this tree is applied to

    :return: Result of the operator call.
    """

def custom_mainloop(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Use a custom Mainloop for this scene

    :return: Result of the operator call.
    """

def custom_mainloop_tree(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Use a custom Mainloop for this scene

    :return: Result of the operator call.
    """

def custom_node_templates(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Load Custom Logic Node Templates

    :return: Result of the operator call.
    """

def edit_custom_node(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Edit Custom Logic Node

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def find_logic_tree(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    tree_name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Edit

    :param tree_name: tree_name, (optional, never None)
    :return: Result of the operator call.
    """

def generate_code(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    uplogic_installed: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Force generation of code, needed only after updating or if encountering issues

    :param uplogic_installed: uplogic_installed, (optional)
    :return: Result of the operator call.
    """

def generate_project(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Generate basic structure for a new project

    :return: Result of the operator call.
    """

def get_owner(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    applied_object: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select the object this tree is applied to

    :param applied_object: applied_object, (optional, never None)
    :return: Result of the operator call.
    """

def install_pyfmodex(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """NOTE: This may take a few seconds and requires internet connection.

    :return: Result of the operator call.
    """

def install_upbge_stubs(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Downloads the latest version of the upbge-stubs module to support autocomplet in your IDE.NOTE: This may take a few seconds and requires internet connection.

    :return: Result of the operator call.
    """

def install_uplogic(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Downloads the latest version of the uplogic module required for running logic nodes.NOTE: This may take a few seconds and requires internet connection.

    :return: Result of the operator call.
    """

def key_selector(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    keycode: str | None = "",
    is_socket: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Undocumented, consider contributing.

    :param keycode: keycode, (optional, never None)
    :param is_socket: is_socket, (optional)
    :return: Result of the operator call.
    """

def load_font(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "",
    filter_glob: str | None = "*.ttf;*.otf;",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Load an image file

    :param filepath: File Path, Filepath used for importing the file (optional, never None)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def load_image(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "",
    filter_glob: str | None = "*.jpg;*.png;*.jpeg;*.JPEG;",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Load an image file

    :param filepath: File Path, Filepath used for importing the file (optional, never None)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def load_sound(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "",
    filter_glob: str | None = "*.wav;*.mp3;*.ogg*",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Load a sound file

    :param filepath: File Path, Filepath used for importing the file (optional, never None)
    :param filter_glob: filter_glob, (optional, never None)
    :return: Result of the operator call.
    """

def move_game_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
    direction: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move Game Property

    :param index: index, (in [-inf, inf], optional)
    :param direction: direction, (optional, never None)
    :return: Result of the operator call.
    """

def node_search(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    node: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Search for registered Logic Nodes

    :param node: node, (optional)
    :return: Result of the operator call.
    """

def open_donate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Please consider supporting this Add-On

    :return: Result of the operator call.
    """

def open_github(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Get involved with development

    :return: Result of the operator call.
    """

def open_upbge_docs(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """UPBGE API Documentation

    :return: Result of the operator call.
    """

def open_upbge_manual(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Manual on engine and node usage

    :return: Result of the operator call.
    """

def pack_new_tree(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    new_tree_name: str | None = "NewTree",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Convert selected Nodes to a new tree. Will be applied to selected object.
    WARNING: All Nodes connected to selection must be selected too

        :param new_tree_name: New Tree Name, (optional, never None)
        :return: Result of the operator call.
    """

def register_custom_node(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    text_name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Register Custom Logic Node

    :param text_name: Node, Register a node defined in a python file (optional)
    :return: Result of the operator call.
    """

def reload_components(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reload all components applied to this object

    :return: Result of the operator call.
    """

def reload_texts(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reload all externally saved scripts

    :return: Result of the operator call.
    """

def remove_custom_node(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove Custom Logic Node

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def remove_game_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove this property

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def remove_global_category(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove a global value category

    :return: Result of the operator call.
    """

def remove_global_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove a value accessible from anywhere

    :return: Result of the operator call.
    """

def remove_logic_tree_property(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    prop_index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove a value accessible from anywhere

    :param prop_index: prop_index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def remove_socket(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove this socket

    :return: Result of the operator call.
    """

def reset_empty_scale(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset the volume scale

    :return: Result of the operator call.
    """

def save_custom_node(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Save Custom Logic Node

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def start_ui_preview(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Show this canvas and its children. Children are determined by connected Widget sockets, Condition sockets dont matter

    :return: Result of the operator call.
    """

def unapply_logic_tree(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    tree_name: str | None = "",
    from_obj_name: str | None = "",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove the tree from the selected objects

    :param tree_name: tree_name, (optional, never None)
    :param from_obj_name: from_obj_name, (optional, never None)
    :return: Result of the operator call.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def bake_keys(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add keyframes on every frame between the selected keyframes

    :return: Result of the operator call.
    """

def blend_offset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Shift selected keys to the value of the neighboring keys as a block

    :param factor: Offset Factor, Control which key to offset towards and how far (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def blend_to_default(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Blend selected keys to their default value from their current position

    :param factor: Factor, How much to blend to the default value (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def blend_to_ease(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Blends keyframes from current state to an ease-in or ease-out curve

    :param factor: Blend, Favor either original data or ease curve (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def blend_to_neighbor(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Blend selected keyframes to their left or right neighbor

    :param factor: Blend, The blend factor with 0 being the current frame (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def breakdown(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move selected keyframes to an inbetween position relative to adjacent keys

    :param factor: Factor, Favor either the left or the right key (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def butterworth_smooth(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    cutoff_frequency: float | None = 3.0,
    filter_order: int | None = 4,
    samples_per_frame: int | None = 1,
    blend: float | None = 1.0,
    blend_in_out: int | None = 1,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Smooth an F-Curve while maintaining the general shape of the curve

    :param cutoff_frequency: Frequency Cutoff (Hz), Lower values give a smoother curve (in [0, inf], optional)
    :param filter_order: Filter Order, Higher values produce a harder frequency cutoff (in [1, 32], optional)
    :param samples_per_frame: Samples per Frame, How many samples to calculate per frame, helps with subframe data (in [1, 64], optional)
    :param blend: Blend, How much to blend to the smoothed curve (in [0, inf], optional)
    :param blend_in_out: Blend In/Out, Linearly blend the smooth data to the border frames of the selection (in [0, inf], optional)
    :return: Result of the operator call.
    """

def clean(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    threshold: float | None = 0.001,
    channels: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Simplify F-Curves by removing closely spaced keyframes

    :param threshold: Threshold, (in [0, inf], optional)
    :param channels: Channels, (optional)
    :return: Result of the operator call.
    """

def click_insert(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    frame: float | None = 1.0,
    value: float | None = 1.0,
    extend: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Insert new keyframe at the cursor position for the active F-Curve

    :param frame: Frame Number, Frame to insert keyframe on (in [-inf, inf], optional)
    :param value: Value, Value for keyframe on (in [-inf, inf], optional)
    :param extend: Extend, Extend selection instead of deselecting everything first (optional)
    :return: Result of the operator call.
    """

def clickselect(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    wait_to_deselect_others: bool | None = False,
    use_select_on_click: bool | None = False,
    mouse_x: int | None = 0,
    mouse_y: int | None = 0,
    extend: bool | None = False,
    deselect_all: bool | None = False,
    column: bool | None = False,
    curves: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframes by clicking on them

    :param wait_to_deselect_others: Wait to Deselect Others, (optional)
    :param use_select_on_click: Act on Click, Instead of selecting on mouse press, wait to see if theres drag event. Otherwise select on mouse release (optional)
    :param mouse_x: Mouse X, (in [-inf, inf], optional)
    :param mouse_y: Mouse Y, (in [-inf, inf], optional)
    :param extend: Extend Select, Toggle keyframe selection instead of leaving newly selected keyframes only (optional)
    :param deselect_all: Deselect On Nothing, Deselect all when nothing under the cursor (optional)
    :param column: Column Select, Select all keyframes that occur on the same frame as the one under the mouse (optional)
    :param curves: Only Curves, Select all the keyframes in the curve (optional)
    :return: Result of the operator call.
    """

def copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy selected keyframes to the internal clipboard

    :return: Result of the operator call.
    """

def cursor_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    frame: float | None = 0.0,
    value: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Interactively set the current frame and value cursor

    :param frame: Frame, (in [-1.04857e+06, 1.04857e+06], optional)
    :param value: Value, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def decimate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    mode: typing.Literal["RATIO", "ERROR"] | None = "RATIO",
    factor: float | None = 0.333333,
    remove_error_margin: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Decimate F-Curves by removing keyframes that influence the curve shape the least

        :param mode: Mode, Which mode to use for decimation (optional)

    RATIO
    Ratio -- Use a percentage to specify how many keyframes you want to remove.

    ERROR
    Error Margin -- Use an error margin to specify how much the curve is allowed to deviate from the original path.
        :param factor: Factor, The ratio of keyframes to remove (in [0, 1], optional)
        :param remove_error_margin: Max Error Margin, How much the new decimated curve is allowed to deviate from the original (in [0, inf], optional)
        :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    confirm: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove all selected keyframes

    :param confirm: Confirm, Prompt for confirmation (optional)
    :return: Result of the operator call.
    """

def driver_delete_invalid(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Delete all visible drivers considered invalid

    :return: Result of the operator call.
    """

def driver_variables_copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy the driver variables of the active driver

    :return: Result of the operator call.
    """

def driver_variables_paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    replace: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add copied driver variables to the active driver

    :param replace: Replace Existing, Replace existing driver variables, instead of just appending to the end of the existing list (optional)
    :return: Result of the operator call.
    """

def duplicate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    mode: Literal[bpy.stub_internal.rna_enums.TransformModeTypeItems]
    | None = "TRANSLATION",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Make a copy of all selected keyframes

    :param mode: Mode, (optional)
    :return: Result of the operator call.
    """

def duplicate_move(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    GRAPH_OT_duplicate: dict[str, typing.Any] | None = {},
    TRANSFORM_OT_translate: dict[str, typing.Any] | None = {},
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Make a copy of all selected keyframes and move them

    :param GRAPH_OT_duplicate: Duplicate Keyframes, Make a copy of all selected keyframes (optional, `bpy.ops.graph.duplicate` keyword arguments)
    :param TRANSFORM_OT_translate: Move, Move selected items (optional, `bpy.ops.transform.translate` keyword arguments)
    :return: Result of the operator call.
    """

def ease(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
    sharpness: float | None = 2.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Align keyframes on a ease-in or ease-out curve

    :param factor: Curve Bend, Defines if the keys should be aligned on an ease-in or ease-out curve (in [-inf, inf], optional)
    :param sharpness: Sharpness, Higher values make the change more abrupt (in [0.001, inf], optional)
    :return: Result of the operator call.
    """

def easing_type(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: Literal[bpy.stub_internal.rna_enums.BeztripleInterpolationEasingItems]
    | None = "AUTO",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set easing type for the F-Curve segments starting from the selected keyframes

    :param type: Type, (optional)
    :return: Result of the operator call.
    """

def equalize_handles(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    side: typing.Literal["LEFT", "RIGHT", "BOTH"] | None = "LEFT",
    handle_length: float | None = 5.0,
    flatten: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Ensure selected keyframes handles have equal length, optionally making them horizontal. Automatic, Automatic Clamped, or Vector handle types will be converted to Aligned

        :param side: Side, Side of the keyframes Bézier handles to affect (optional)

    LEFT
    Left -- Equalize selected keyframes left handles.

    RIGHT
    Right -- Equalize selected keyframes right handles.

    BOTH
    Both -- Equalize both of a keyframes handles.
        :param handle_length: Handle Length, Length to make selected keyframes Bézier handles (in [0.1, inf], optional)
        :param flatten: Flatten, Make the values of the selected keyframes handles the same as their respective keyframes (optional)
        :return: Result of the operator call.
    """

def euler_filter(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Fix large jumps and flips in the selected Euler Rotation F-Curves arising from rotation values being clipped when baking physics

    :return: Result of the operator call.
    """

def extrapolation_type(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["CONSTANT", "LINEAR", "MAKE_CYCLIC", "CLEAR_CYCLIC"]
    | None = "CONSTANT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set extrapolation mode for selected F-Curves

        :param type: Type, (optional)

    CONSTANT
    Constant Extrapolation -- Values on endpoint keyframes are held.

    LINEAR
    Linear Extrapolation -- Straight-line slope of end segments are extended past the endpoint keyframes.

    MAKE_CYCLIC
    Make Cyclic (F-Modifier) -- Add Cycles F-Modifier if one does not exist already.

    CLEAR_CYCLIC
    Clear Cyclic (F-Modifier) -- Remove Cycles F-Modifier if not needed anymore.
        :return: Result of the operator call.
    """

def fmodifier_add(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: Literal[bpy.stub_internal.rna_enums.FmodifierTypeItems] | None = "NULL",
    only_active: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add F-Modifier to the active/selected F-Curves

    :param type: Type, (optional)
    :param only_active: Only Active, Only add F-Modifier to active F-Curve (optional)
    :return: Result of the operator call.
    """

def fmodifier_copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy the F-Modifier(s) of the active F-Curve

    :return: Result of the operator call.
    """

def fmodifier_paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    only_active: bool | None = False,
    replace: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add copied F-Modifiers to the selected F-Curves

    :param only_active: Only Active, Only paste F-Modifiers on active F-Curve (optional)
    :param replace: Replace Existing, Replace existing F-Modifiers, instead of just appending to the end of the existing list (optional)
    :return: Result of the operator call.
    """

def frame_jump(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Place the cursor on the midpoint of selected keyframes

    :return: Result of the operator call.
    """

def gaussian_smooth(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 1.0,
    sigma: float | None = 0.33,
    filter_width: int | None = 6,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Smooth the curve using a Gaussian filter

    :param factor: Factor, How much to blend to the default value (in [0, inf], optional)
    :param sigma: Sigma, The shape of the gaussian distribution, lower values make it sharper (in [0.001, inf], optional)
    :param filter_width: Filter Width, How far to each side the operator will average the key values (in [1, 64], optional)
    :return: Result of the operator call.
    """

def ghost_curves_clear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear F-Curve snapshots (Ghosts) for active Graph Editor

    :return: Result of the operator call.
    """

def ghost_curves_create(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Create snapshot (Ghosts) of selected F-Curves as background aid for active Graph Editor

    :return: Result of the operator call.
    """

def handle_type(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: Literal[bpy.stub_internal.rna_enums.KeyframeHandleTypeItems] | None = "FREE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set type of handle for selected keyframes

    :param type: Type, (optional)
    :return: Result of the operator call.
    """

def hide(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    unselected: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Hide selected curves from Graph Editor view

    :param unselected: Unselected, Hide unselected rather than selected curves (optional)
    :return: Result of the operator call.
    """

def interpolation_type(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: Literal[bpy.stub_internal.rna_enums.BeztripleInterpolationModeItems]
    | None = "CONSTANT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set interpolation mode for the F-Curve segments starting from the selected keyframes

    :param type: Type, (optional)
    :return: Result of the operator call.
    """

def keyframe_insert(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["ALL", "SEL", "ACTIVE", "CURSOR_ACTIVE", "CURSOR_SEL"]
    | None = "ALL",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Insert keyframes for the specified channels

        :param type: Type, (optional)

    ALL
    All Channels -- Insert a keyframe on all visible and editable F-Curves using each curves current value.

    SEL
    Only Selected Channels -- Insert a keyframe on selected F-Curves using each curves current value.

    ACTIVE
    Only Active F-Curve -- Insert a keyframe on the active F-Curve using the curves current value.

    CURSOR_ACTIVE
    Active Channels at Cursor -- Insert a keyframe for the active F-Curve at the cursor point.

    CURSOR_SEL
    Selected Channels at Cursor -- Insert a keyframe for selected F-Curves at the cursor point.
        :return: Result of the operator call.
    """

def keyframe_jump(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    next: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Jump to previous/next keyframe

    :param next: Next Keyframe, (optional)
    :return: Result of the operator call.
    """

def keys_to_samples(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Convert selected channels to an uneditable set of samples to save storage space

    :return: Result of the operator call.
    """

def match_slope(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Blend selected keys to the slope of neighboring ones

    :param factor: Factor, Defines which keys to use as slope and how much to blend towards them (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def mirror(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal["CFRA", "VALUE", "YAXIS", "XAXIS", "MARKER"] | None = "CFRA",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Flip selected keyframes over the selected mirror line

        :param type: Type, (optional)

    CFRA
    By Times Over Current Frame -- Flip times of selected keyframes using the current frame as the mirror line.

    VALUE
    By Values Over Cursor Value -- Flip values of selected keyframes using the cursor value (Y/Horizontal component) as the mirror line.

    YAXIS
    By Times Over Zero Time -- Flip times of selected keyframes, effectively reversing the order they appear in.

    XAXIS
    By Values Over Zero Value -- Flip values of selected keyframes (i.e. negative values become positive, and vice versa).

    MARKER
    By Times Over First Selected Marker -- Flip times of selected keyframes using the first selected marker as the reference point.
        :return: Result of the operator call.
    """

def paste(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    offset: Literal[bpy.stub_internal.rna_enums.KeyframePasteOffsetItems]
    | None = "START",
    value_offset: Literal[bpy.stub_internal.rna_enums.KeyframePasteOffsetValueItems]
    | None = "NONE",
    merge: Literal[bpy.stub_internal.rna_enums.KeyframePasteMergeItems] | None = "MIX",
    flipped: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Paste keyframes from the internal clipboard for the selected channels, starting on the current frame

    :param offset: Frame Offset, Paste time offset of keys (optional)
    :param value_offset: Value Offset, Paste keys with a value offset (optional)
    :param merge: Type, Method of merging pasted keys and existing (optional)
    :param flipped: Flipped, Paste keyframes from mirrored bones if they exist (optional)
    :return: Result of the operator call.
    """

def previewrange_set(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set Preview Range based on range of selected keyframes

    :return: Result of the operator call.
    """

def push_pull(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 1.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Exaggerate or minimize the value of the selected keys

    :param factor: Factor, Control how far to push or pull the keys (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def reveal(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    select: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Make previously hidden curves visible again in Graph Editor view

    :param select: Select, (optional)
    :return: Result of the operator call.
    """

def samples_to_keys(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Convert selected channels from samples to keyframes

    :return: Result of the operator call.
    """

def scale_average(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 1.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Scale selected key values by their combined average

    :param factor: Scale Factor, The scale factor applied to the curve segments (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def scale_from_neighbor(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
    anchor: typing.Literal["LEFT", "RIGHT"] | None = "LEFT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Increase or decrease the value of selected keys in relationship to the neighboring one

    :param factor: Factor, The factor to scale keys with (in [-inf, inf], optional)
    :param anchor: Reference Key, Which end of the segment to use as a reference to scale from (optional)
    :return: Result of the operator call.
    """

def select_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    action: typing.Literal["TOGGLE", "SELECT", "DESELECT", "INVERT"] | None = "TOGGLE",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Toggle selection of all keyframes

        :param action: Action, Selection action to execute (optional)

    TOGGLE
    Toggle -- Toggle selection for all elements.

    SELECT
    Select -- Select all elements.

    DESELECT
    Deselect -- Deselect all elements.

    INVERT
    Invert -- Invert selection of all elements.
        :return: Result of the operator call.
    """

def select_box(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    axis_range: bool | None = False,
    include_handles: bool | None = True,
    tweak: bool | None = False,
    use_curve_selection: bool | None = True,
    xmin: int | None = 0,
    xmax: int | None = 0,
    ymin: int | None = 0,
    ymax: int | None = 0,
    wait_for_input: bool | None = True,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select all keyframes within the specified region

        :param axis_range: Axis Range, (optional)
        :param include_handles: Include Handles, Are handles tested individually against the selection criteria, independently from their keys. When unchecked, handles are (de)selected in unison with their keys (optional)
        :param tweak: Tweak, Operator has been activated using a click-drag event (optional)
        :param use_curve_selection: Select Curves, Allow selecting all the keyframes of a curve by selecting the calculated F-curve (optional)
        :param xmin: X Min, (in [-inf, inf], optional)
        :param xmax: X Max, (in [-inf, inf], optional)
        :param ymin: Y Min, (in [-inf, inf], optional)
        :param ymax: Y Max, (in [-inf, inf], optional)
        :param wait_for_input: Wait for Input, (optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :return: Result of the operator call.
    """

def select_circle(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    x: int | None = 0,
    y: int | None = 0,
    radius: int | None = 25,
    wait_for_input: bool | None = True,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
    include_handles: bool | None = True,
    use_curve_selection: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframe points using circle selection

        :param x: X, (in [-inf, inf], optional)
        :param y: Y, (in [-inf, inf], optional)
        :param radius: Radius, (in [1, inf], optional)
        :param wait_for_input: Wait for Input, (optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :param include_handles: Include Handles, Are handles tested individually against the selection criteria, independently from their keys. When unchecked, handles are (de)selected in unison with their keys (optional)
        :param use_curve_selection: Select Curves, Allow selecting all the keyframes of a curve by selecting the curve itself (optional)
        :return: Result of the operator call.
    """

def select_column(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    mode: typing.Literal["KEYS", "CFRA", "MARKERS_COLUMN", "MARKERS_BETWEEN"]
    | None = "KEYS",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select all keyframes on the specified frame(s)

    :param mode: Mode, (optional)
    :return: Result of the operator call.
    """

def select_key_handles(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    left_handle_action: typing.Literal["SELECT", "DESELECT", "KEEP"] | None = "SELECT",
    right_handle_action: typing.Literal["SELECT", "DESELECT", "KEEP"] | None = "SELECT",
    key_action: typing.Literal["SELECT", "DESELECT", "KEEP"] | None = "KEEP",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """For selected keyframes, select/deselect any combination of the key itself and its handles

        :param left_handle_action: Left Handle, Effect on the left handle (optional)

    SELECT
    Select.

    DESELECT
    Deselect.

    KEEP
    Keep -- Leave as is.
        :param right_handle_action: Right Handle, Effect on the right handle (optional)

    SELECT
    Select.

    DESELECT
    Deselect.

    KEEP
    Keep -- Leave as is.
        :param key_action: Key, Effect on the key itself (optional)

    SELECT
    Select.

    DESELECT
    Deselect.

    KEEP
    Keep -- Leave as is.
        :return: Result of the operator call.
    """

def select_lasso(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    path=None,
    use_smooth_stroke: bool | None = False,
    smooth_stroke_factor: float | None = 0.75,
    smooth_stroke_radius: int | None = 35,
    mode: typing.Literal["SET", "ADD", "SUB"] | None = "SET",
    include_handles: bool | None = True,
    use_curve_selection: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframe points using lasso selection

        :param path: Path, (optional)
        :param use_smooth_stroke: Stabilize Stroke, Selection lags behind mouse and follows a smoother path (optional)
        :param smooth_stroke_factor: Smooth Stroke Factor, Higher values give a smoother stroke (in [0.5, 0.99], optional)
        :param smooth_stroke_radius: Smooth Stroke Radius, Minimum distance from last point before selection continues (in [10, 200], optional)
        :param mode: Mode, (optional)

    SET
    Set -- Set a new selection.

    ADD
    Extend -- Extend existing selection.

    SUB
    Subtract -- Subtract existing selection.
        :param include_handles: Include Handles, Are handles tested individually against the selection criteria, independently from their keys. When unchecked, handles are (de)selected in unison with their keys (optional)
        :param use_curve_selection: Select Curves, Allow selecting all the keyframes of a curve by selecting the curve itself (optional)
        :return: Result of the operator call.
    """

def select_leftright(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    mode: typing.Literal["CHECK", "LEFT", "RIGHT"] | None = "CHECK",
    extend: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframes to the left or the right of the current frame

    :param mode: Mode, (optional)
    :param extend: Extend Select, (optional)
    :return: Result of the operator call.
    """

def select_less(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Deselect keyframes on ends of selection islands

    :return: Result of the operator call.
    """

def select_linked(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframes occurring in the same F-Curves as selected ones

    :return: Result of the operator call.
    """

def select_more(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Select keyframes beside already selected ones

    :return: Result of the operator call.
    """

def shear(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    factor: float | None = 0.0,
    direction: typing.Literal["FROM_LEFT", "FROM_RIGHT"] | None = "FROM_LEFT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Affect the value of the keys linearly, keeping the same relationship between them using either the left or the right key as reference

        :param factor: Shear Factor, The amount of shear to apply (in [-inf, inf], optional)
        :param direction: Direction, Which end of the segment to use as a reference to shear from (optional)

    FROM_LEFT
    From Left -- Shear the keys using the left key as reference.

    FROM_RIGHT
    From Right -- Shear the keys using the right key as reference.
        :return: Result of the operator call.
    """

def smooth(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Apply weighted moving means to make selected F-Curves less bumpy

    :return: Result of the operator call.
    """

def snap(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    type: typing.Literal[
        "CFRA",
        "VALUE",
        "NEAREST_FRAME",
        "NEAREST_SECOND",
        "NEAREST_MARKER",
        "HORIZONTAL",
    ]
    | None = "CFRA",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Snap selected keyframes to the chosen times/values

        :param type: Type, (optional)

    CFRA
    Selection to Current Frame -- Snap selected keyframes to the current frame.

    VALUE
    Selection to Cursor Value -- Set values of selected keyframes to the cursor value (Y/Horizontal component).

    NEAREST_FRAME
    Selection to Nearest Frame -- Snap selected keyframes to the nearest (whole) frame (use to fix accidental subframe offsets).

    NEAREST_SECOND
    Selection to Nearest Second -- Snap selected keyframes to the nearest second.

    NEAREST_MARKER
    Selection to Nearest Marker -- Snap selected keyframes to the nearest marker.

    HORIZONTAL
    Flatten Handles -- Flatten handles for a smoother transition.
        :return: Result of the operator call.
    """

def snap_cursor_value(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Place the cursor value on the average value of selected keyframes

    :return: Result of the operator call.
    """

def sound_to_samples(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    filepath: str | None = "",
    check_existing: bool | None = False,
    filter_blender: bool | None = False,
    filter_backup: bool | None = False,
    filter_image: bool | None = False,
    filter_movie: bool | None = True,
    filter_python: bool | None = False,
    filter_font: bool | None = False,
    filter_sound: bool | None = True,
    filter_text: bool | None = False,
    filter_archive: bool | None = False,
    filter_btx: bool | None = False,
    filter_alembic: bool | None = False,
    filter_usd: bool | None = False,
    filter_obj: bool | None = False,
    filter_volume: bool | None = False,
    filter_folder: bool | None = True,
    filter_blenlib: bool | None = False,
    filemode: int | None = 9,
    show_multiview: bool | None = False,
    use_multiview: bool | None = False,
    display_type: typing.Literal[
        "DEFAULT", "LIST_VERTICAL", "LIST_HORIZONTAL", "THUMBNAIL"
    ]
    | None = "DEFAULT",
    sort_method: str | None = "",
    low: float | None = 0.0,
    high: float | None = 100000.0,
    attack: float | None = 0.005,
    release: float | None = 0.2,
    threshold: float | None = 0.0,
    use_accumulate: bool | None = False,
    use_additive: bool | None = False,
    use_square: bool | None = False,
    sthreshold: float | None = 0.1,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Bakes a sound wave to samples on selected channels

        :param filepath: File Path, Path to file (optional, never None)
        :param check_existing: Check Existing, Check and warn on overwriting existing files (optional)
        :param filter_blender: Filter .blend files, (optional)
        :param filter_backup: Filter backup .blend files, (optional)
        :param filter_image: Filter image files, (optional)
        :param filter_movie: Filter movie files, (optional)
        :param filter_python: Filter Python files, (optional)
        :param filter_font: Filter font files, (optional)
        :param filter_sound: Filter sound files, (optional)
        :param filter_text: Filter text files, (optional)
        :param filter_archive: Filter archive files, (optional)
        :param filter_btx: Filter btx files, (optional)
        :param filter_alembic: Filter Alembic files, (optional)
        :param filter_usd: Filter USD files, (optional)
        :param filter_obj: Filter OBJ files, (optional)
        :param filter_volume: Filter OpenVDB volume files, (optional)
        :param filter_folder: Filter folders, (optional)
        :param filter_blenlib: Filter Blender IDs, (optional)
        :param filemode: File Browser Mode, The setting for the file browser mode to load a .blend file, a library or a special file (in [1, 9], optional)
        :param show_multiview: Enable Multi-View, (optional)
        :param use_multiview: Use Multi-View, (optional)
        :param display_type: Display Type, (optional)

    DEFAULT
    Default -- Automatically determine display type for files.

    LIST_VERTICAL
    Short List -- Display files as short list.

    LIST_HORIZONTAL
    Long List -- Display files as a detailed list.

    THUMBNAIL
    Thumbnails -- Display files as thumbnails.
        :param sort_method: File sorting mode, (optional)
        :param low: Lowest Frequency, Cutoff frequency of a high-pass filter that is applied to the audio data (in [0, 100000], optional)
        :param high: Highest Frequency, Cutoff frequency of a low-pass filter that is applied to the audio data (in [0, 100000], optional)
        :param attack: Attack Time, Value for the envelope calculation that tells how fast the envelope can rise (the lower the value the steeper it can rise) (in [0, 2], optional)
        :param release: Release Time, Value for the envelope calculation that tells how fast the envelope can fall (the lower the value the steeper it can fall) (in [0, 5], optional)
        :param threshold: Threshold, Minimum amplitude value needed to influence the envelope (in [0, 1], optional)
        :param use_accumulate: Accumulate, Only the positive differences of the envelope amplitudes are summarized to produce the output (optional)
        :param use_additive: Additive, The amplitudes of the envelope are summarized (or, when Accumulate is enabled, both positive and negative differences are accumulated) (optional)
        :param use_square: Square, The output is a square curve (negative values always result in -1, and positive ones in 1) (optional)
        :param sthreshold: Square Threshold, Square only: all values with an absolute amplitude lower than that result in 0 (in [0, 1], optional)
        :return: Result of the operator call.
    """

def time_offset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    frame_offset: float | None = 0.0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Shifts the value of selected keys in time

    :param frame_offset: Frame Offset, How far in frames to offset the animation (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def view_all(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    include_handles: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset viewable area to show full keyframe range

    :param include_handles: Include Handles, Include handles of keyframes when calculating extents (optional)
    :return: Result of the operator call.
    """

def view_frame(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move the view to the current frame

    :return: Result of the operator call.
    """

def view_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    include_handles: bool | None = True,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset viewable area to show selected keyframe range

    :param include_handles: Include Handles, Include handles of keyframes when calculating extents (optional)
    :return: Result of the operator call.
    """



"""
This module provides utilities for textures.

"""

import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types
import gpu.types

def from_external_memory(
    handle: int,
    handle_type: str,
    format: typing.Literal[
        "RGBA8UI",
        "RGBA8I",
        "RGBA8",
        "RGBA32UI",
        "RGBA32I",
        "RGBA32F",
        "RGBA16UI",
        "RGBA16I",
        "RGBA16F",
        "RGBA16",
        "RG8UI",
        "RG8I",
        "RG8",
        "RG32UI",
        "RG32I",
        "RG32F",
        "RG16UI",
        "RG16I",
        "RG16F",
        "RG16",
        "R8UI",
        "R8I",
        "R8",
        "R32UI",
        "R32I",
        "R32F",
        "R16UI",
        "R16I",
        "R16F",
        "R16",
        "R11F_G11F_B10F",
        "DEPTH32F_STENCIL8",
        "DEPTH24_STENCIL8",
        "SRGB8_A8",
        "RGB16F",
        "SRGB8_A8_DXT1",
        "SRGB8_A8_DXT3",
        "SRGB8_A8_DXT5",
        "RGBA8_DXT1",
        "RGBA8_DXT3",
        "RGBA8_DXT5",
        "DEPTH_COMPONENT32F",
        "DEPTH_COMPONENT24",
        "DEPTH_COMPONENT16",
    ],
    width: int,
    height: int,
    stride: int,
) -> None | gpu.types.GPUTexture:
    """Create a `gpu.types.GPUTexture` from an externally-owned GPU memory handle.
    Zero-copy Vulkan external memory import. Requires the Vulkan backend and the matching
    device extension (VK_KHR_external_memory_fd on Linux,
    VK_KHR_external_memory_win32 on Windows).
    Returns None if the extension is unavailable or import fails.

        :param handle: DMA-BUF fd (Linux) or Win32 HANDLE (Windows) cast to int.
        :param handle_type: External memory handle type string.
    DMA_BUF    VK_EXTERNAL_MEMORY_HANDLE_TYPE_DMA_BUF_BIT_EXT (Linux).
    D3D11_KMT  VK_EXTERNAL_MEMORY_HANDLE_TYPE_D3D11_TEXTURE_KMT_BIT (Windows).
        :param format: Internal GPU texture format string.
        :param width: Texture width in pixels.
        :param height: Texture height in pixels.
        :param stride: Row stride in bytes.
        :return: The imported texture, or None on failure.
    """

def from_image(image: bpy.types.Image) -> gpu.types.GPUTexture:
    """Get GPUTexture corresponding to an Image data-block. The GPUTexture memory is shared with Blender.
    Note: Colors read from the texture will be in scene linear color space and have premultiplied or straight alpha matching the image alpha mode.

        :param image: The Image data-block.
        :return: The GPUTexture used by the image.
    """

def set_image_texture(
    image: bpy.types.Image, texture: None | gpu.types.GPUTexture
) -> None:
    """Override the GPU texture used by a `bpy.types.Image` datablock.
    Material Image Texture nodes will sample from texture instead of
    uploading pixel data from CPU memory.
    Pass None to clear the override and restore normal behaviour.

        :param image: The Image datablock to override.
        :param texture: Texture to use, or None to clear.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.stub_internal.rna_enums

def add_target(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add a target to the constraint

    :return: Result of the operator call.
    """

def apply(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
    report: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Apply constraint and remove from the stack

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :param report: Report, Create a notification after the operation (optional)
        :return: Result of the operator call.
    """

def childof_clear_inverse(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear inverse correction for Child Of constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def childof_set_inverse(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set inverse correction for Child Of constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def copy(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
    report: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Duplicate constraint at the same position in the stack

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :param report: Report, Create a notification after the operation (optional)
        :return: Result of the operator call.
    """

def copy_to_selected(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Copy constraint to other selected objects/bones

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def delete(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
    report: bool | None = False,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove constraint from constraint stack

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :param report: Report, Create a notification after the operation (optional)
        :return: Result of the operator call.
    """

def disable_keep_transform(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set the influence of this constraint to zero while trying to maintain the objects transformation. Other active constraints can still influence the final transformation

    :return: Result of the operator call.
    """

def followpath_path_animate(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
    frame_start: int | None = 1,
    length: int | None = 100,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Add default animation for path used by constraint if it isnt animated already

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :param frame_start: Start Frame, First frame of path animation (in [-1048574, 1048574], optional)
        :param length: Length, Number of frames that path animation should take (in [0, 1048574], optional)
        :return: Result of the operator call.
    """

def limitdistance_reset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset limiting distance for Limit Distance Constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def move_down(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move constraint down in constraint stack

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def move_to_index(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Change the constraints position in the list so it evaluates after the set number of others

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :param index: Index, The index to move the constraint to (in [0, inf], optional)
        :return: Result of the operator call.
    """

def move_up(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Move constraint up in constraint stack

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def normalize_target_weights(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Normalize weights of all target bones

    :return: Result of the operator call.
    """

def objectsolver_clear_inverse(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Clear inverse correction for Object Solver constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def objectsolver_set_inverse(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Set inverse correction for Object Solver constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """

def remove_target(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    index: int | None = 0,
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Remove the target from the constraint

    :param index: index, (in [-inf, inf], optional)
    :return: Result of the operator call.
    """

def stretchto_reset(
    execution_context: int | str | None = None,
    undo: bool | None = None,
    /,
    *,
    constraint: str | None = "",
    owner: typing.Literal["OBJECT", "BONE"] | None = "OBJECT",
) -> set[Literal[bpy.stub_internal.rna_enums.OperatorReturnItems]]:
    """Reset original length of bone for Stretch To Constraint

        :param constraint: Constraint, Name of the constraint to edit (optional, never None)
        :param owner: Owner, The owner of this constraint (optional)

    OBJECT
    Object -- Edit a constraint on the active object.

    BONE
    Bone -- Edit a constraint on the active bone.
        :return: Result of the operator call.
    """
